﻿/*
 * Created by SharpDevelop.
 * User: jt
 * Date: 24-10-2010
 * Time: 16:18
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace HovText
{
	// Must inherit Control, not Component, in order to have Handle
	[DefaultEvent("ClipboardChanged")]
	public partial class ClipboardMonitor : Control 
	{
	    IntPtr nextClipboardViewer;
	
	    bool removed = false;
	    
	    public ClipboardMonitor()
	    {
	        this.BackColor = Color.Red;
	        this.Visible = false;
	
	        this.CreateHandle();
	        nextClipboardViewer = (IntPtr)SetClipboardViewer((int)this.Handle);
	    }
	
	    /// <summary>
	    /// Clipboard contents changed.
	    /// </summary>
	    public event EventHandler<ClipboardChangedEventArgs> ClipboardChanged;
	
		protected override void DestroyHandle()
		{
			Stop();
			base.DestroyHandle();
		}
		
	    protected override void Dispose(bool disposing)
	    {
	    	Stop();
	    }
	
	    public void Stop()
	    {
			if (!removed)
	        	ChangeClipboardChain(this.Handle, nextClipboardViewer);
			removed = true;
	    }
	
	    
	    
	    [DllImport("User32.dll")]
	    protected static extern int SetClipboardViewer(int hWndNewViewer);
	
	    [DllImport("User32.dll", CharSet = CharSet.Auto)]
	    public static extern bool ChangeClipboardChain(IntPtr hWndRemove, IntPtr hWndNewNext);
	
		[DllImport("user32.dll", CharSet = CharSet.Auto)]
	    public static extern int SendMessage(IntPtr hwnd, int wMsg, IntPtr wParam, IntPtr lParam);	
	    
		[DllImport("user32.dll", CharSet = CharSet.Auto)]
	    public static extern int PostMessage(IntPtr hwnd, int wMsg, IntPtr wParam, IntPtr lParam);
	    
	    protected override void WndProc(ref System.Windows.Forms.Message m)
	    {
	        // defined in winuser.h
	        const int WM_DRAWCLIPBOARD = 0x0308;
	        const int WM_CHANGECBCHAIN = 0x030D;
			//const int WM_USER = 0x0400;
			const int WM_CBCHANGE = 0x0400;
			switch (m.Msg)
	        {
	            case WM_DRAWCLIPBOARD:
					SendMessage(nextClipboardViewer, m.Msg, m.WParam, m.LParam);
					PostMessage(this.Handle, WM_CBCHANGE, IntPtr.Zero, IntPtr.Zero);
					//OnClipboardChanged();
	                break;
	
	            case WM_CBCHANGE:
					OnClipboardChanged();
	                break;                
	            case WM_CHANGECBCHAIN:
	                if (m.WParam == nextClipboardViewer)
	                    nextClipboardViewer = m.LParam;
	                else
	                    SendMessage(nextClipboardViewer, m.Msg, m.WParam, m.LParam);
	                break;
	
	            default:
	                base.WndProc(ref m);
	                break;
	        }
	    }
	
	    void OnClipboardChanged()
	    {
	        try
	        {
	            IDataObject iData = Clipboard.GetDataObject();
	            if (ClipboardChanged != null)
	            {
	                ClipboardChanged(this, new ClipboardChangedEventArgs(iData));
	            }
	        }
	        catch (Exception e)
	        {
	            // Swallow or pop-up, not sure
	            // Trace.Write(e.ToString());
	            MessageBox.Show(e.ToString());
	        }
	    }
	}
		
	public class ClipboardChangedEventArgs : EventArgs
	{
	    public readonly IDataObject DataObject;
	
	    public ClipboardChangedEventArgs(IDataObject dataObject)
	    {
	        DataObject = dataObject;
	    }
	}
}
